{
	TFlightJoyStick 1.0 By Paul van Dinther Copyright Diprode 18-06-2000
	e-mail: paul@diprode.com
	Website: http://www.diprode.com

	TFlightJoyStick is a special joystick component used in conjuction with the
	Aircraft Library from Diprode. TFlightJoyStick is also usable as a stand-
	alone component. Only the basic joystick functionality is implemented.

	You can use the component in AutoPolling mode or simply retrieve the joystick
	values whenever you need them. The events from TFlightJoyStick are only fired
	when AutoPolling is True and the actual joystick values have changed since the
	last polling cycle.

	You can control the polling cycle interval with the PollingTime property
	expressed in milliseconds.

	If you use TflightJoyStick inconjunction with the TFlightDynamics or TAircraft
	components then simply connecting the joystick component is sufficient.

	TFlightJoyStick also works when there is no joystick attached on the PC. Simply
	Click and hold the Left mouse button on the visible control and start steering.

}
unit FlightJoyStick;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	ExtCtrls, MMsystem;

const
	BTN_SIZE: integer = 8;
	MARKER_SIZE: Integer = 6;
	MARGIN: Integer = 2;
	ARROW_SIZE = 4;

type
	TJoyStickInfo = record
		X_Axis: single;
		Y_Axis: single;
		Z_Axis: single;
		Button1: Boolean;
		Button2: Boolean;
		Button3: Boolean;
		Button4: Boolean;
	end;
	TJoyControlMode = (jcStick, jcXYMouse, jcZMouse, jcbutton1, jcbutton2, jcbutton3, jcbutton4, jcNone);
	TPosChangeEvent = procedure (Sender: TObject; JoyStickInfo: TJoyStickInfo) of object;
	TFlightJoyStick = class(TCustomControl)
	private
	FStickyButtons: Boolean;
	FStickEnabled: Boolean;
	FOldCursorPos: TPoint;
	FCurrentMode: TJoyControlMode;
	FJoyStickInfo: TJoyStickInfo;
	FXCursor: Integer;
	FYCursor: Integer;
	FZCursor: Integer;
	FCaptured: boolean;
	FJoyEnabledRect: TRect;
	FXAxisRect: TRect;
	FYAxisRect: TRect;
	FZAxisRect: TRect;
	FXYAxisRect: TRect;
	FButton1Rect: TRect;
	FButton2Rect: TRect;
	FButton3Rect: TRect;
	FButton4Rect: TRect;
	FOnXYChange: TPosChangeEvent;
	FOnZChange: TPosChangeEvent;
	FOnButton1Down: TNotifyEvent;
	FOnButton2Down: TNotifyEvent;
	FOnButton3Down: TNotifyEvent;
	FOnButton4Down: TNotifyEvent;
	FOnButton1Up: TNotifyEvent;
	FOnButton2Up: TNotifyEvent;
	FOnButton3Up: TNotifyEvent;
	FOnButton4Up: TNotifyEvent;
	FButtons: Integer;
	FShowZAxis: Boolean;
	FShowButtons: Boolean;
	FSpacing: Integer;
	FAutoPolling: Boolean;
	FZRulerColor: TColor;
	FRulerColor: TColor;
	FButtonColor: TColor;
	FButtonEdgeColor: TColor;
	FZPointerColor: TColor;
	FPointerColor: TColor;
	FEnabledColor: TColor;
	FPollingInterval: Integer;
	FJoyCapability: Tjoycaps;
	function GetJoyStickInfo: TJoystickInfo;
	function GetXAxis: Single;
	function GetYAxis: Single;
	function GetZAxis: Single;
	function GetButton1: Boolean;
	function GetButton2: Boolean;
	function GetButton3: Boolean;
	function GetButton4: Boolean;
	procedure SetStickEnabled(PValue: Boolean);
	procedure SetAutoPolling(PValue: Boolean);
	procedure SetShowZAxis(PValue: Boolean);
	procedure SetShowButtons(Pvalue: Boolean);
	procedure SetSpacing(PValue: Integer);
	procedure SetZRulerColor(Pvalue: TColor);
	procedure SetRulerColor(Pvalue: TColor);
	procedure SetEnabledColor(Pvalue: TColor);
	procedure SetButtonColor(Pvalue: TColor);
	procedure SetButtonEdgeColor(PValue: TColor);
	procedure SetZPointerColor(Pvalue: TColor);
	procedure SetPointerColor(Pvalue: TColor);
	procedure SetPollingInterval(Pvalue: Integer);
	procedure MMJOY1BUTTONDOWN(var Message: TMessage); message MM_JOY1BUTTONDOWN;
	procedure MMJOY1BUTTONUP(var Message: TMessage); message MM_JOY1BUTTONUP;
	procedure MMJOY1ZMOVE(var Message: TMessage); message MM_JOY1ZMOVE;
	procedure MMJOY1MOVE(var Message: TMessage); message MM_JOY1MOVE;
	procedure WMSIZE(var Message: TWMSIZE); message WM_Size;
	procedure CalcAreas;
	procedure paintButton(PID: Integer; PON: Boolean);
	procedure PaintDownArrow(PPoint: Tpoint);
	procedure EraseDownArrow(PPoint: Tpoint);
	procedure PaintRightArrow(PPoint: Tpoint);
	procedure EraseRightArrow(PPoint: Tpoint);
	procedure PaintLeftArrow(PPoint: Tpoint);
	procedure EraseLeftArrow(PPoint: Tpoint);
	procedure PaintXPointer(PForcePaint: Boolean);
	procedure PaintYPointer(PForcePaint: Boolean);
	procedure PaintZPointer(PForcePaint: Boolean);
	procedure PaintStickEnabled(PEnabled: Boolean);
	procedure ButtonEvent(PButtonID: Integer; PDown: Boolean);
	protected
		{ Protected declarations }
		procedure paint; override;
		procedure SetParent(AParent: TWinControl); override;
		procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
		procedure MouseDown(Button: TMouseButton;	Shift: TShiftState; X, Y: Integer); override;
		procedure MouseUp(Button: TMouseButton;	Shift: TShiftState; X, Y: Integer); override;
	public
		constructor Create(AOwner: TComponent); override;
		destructor Destroy; override;
		procedure ReadJoyStick;
		procedure PaintJoystickInfo;
	published
		property Stickybuttons: Boolean read FStickyButtons write FStickyButtons;
		property StickEnabled: Boolean read FStickEnabled write SetStickEnabled;
		property JoystickInfo: TJoyStickInfo read GetJoystickInfo;
		property X_Axis: single read GetXAxis;
		property Y_Axis: single read GetYAxis;
		property Z_Axis: single read GetZAxis;
		property Button1: boolean read GetButton1;
		property Button2: boolean read GetButton2;
		property Button3: boolean read GetButton3;
		property button4: boolean read GetButton4;
		property ShowZAxis: Boolean read FShowZAxis write SetShowZAxis;
		Property ShowButtons: Boolean read FShowButtons write SetShowButtons;
		property Spacing: Integer read FSpacing write SetSpacing;
		property AutoPolling: Boolean read FAutoPolling write SetAutoPolling;
		property PollingInterval: Integer read FPollingInterval write SetPollingInterval;
		property OnXYChange: TPosChangeEvent read FOnXYChange write FOnXYChange;
		property OnZChange: TPosChangeEvent read FOnZChange write FOnZChange;
		property OnButton1Down: TNotifyEvent read FOnButton1Down write FOnButton1Down;
		property OnButton2Down: TNotifyEvent read FOnButton2Down write FOnButton2Down;
		property OnButton3Down: TNotifyEvent read FOnButton3Down write FOnButton3Down;
		property OnButton4Down: TNotifyEvent read FOnButton4Down write FOnButton4Down;
		property OnButton1Up: TNotifyEvent read FOnButton1Up write FOnButton1Up;
		property OnButton2Up: TNotifyEvent read FOnButton2Up write FOnButton2Up;
		property OnButton3Up: TNotifyEvent read FOnButton3Up write FOnButton3Up;
		property OnButton4Up: TNotifyEvent read FOnButton4Up write FOnButton4Up;
		property ZRulerColor: TColor read FZRulerColor write SetZRulerColor;
		property RulerColor: TColor read FRulerColor write SetRulerColor;
		property EnabledColor: TColor read FEnabledColor write SetEnabledColor;
		property ButtonColor: TColor read FButtonColor write SetButtonColor;
		property ButtonEdgeColor: TColor read FButtonEdgeColor write SetButtonEdgeColor;
		property ZPointerColor: TColor read FZPointerColor write SetZPointerColor;
		property PointerColor: TColor read FPointerColor write SetPointerColor;
		property Color;
		property Visible;
		property Enabled;
		property TabStop;
		property TabOrder;
	end;

procedure Register;

implementation

procedure Register;
begin
	RegisterComponents('Diprode', [TFlightJoyStick]);
end;

constructor TFlightJoyStick.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
//ControlStyle := ControlStyle + [csCaptureMouse];
FStickEnabled := joyGetDevCaps(joystickid1, @FJoyCapability,500) = JOYERR_NOERROR;
FAutoPolling := True;
FPollingInterval := 50;
FZRulerColor := clSilver;
FRulerColor := clWhite;
FButtonColor := clRed;
FButtonEdgeColor := clWhite;
FZPointerColor := clWhite;
FEnabledColor := clGreen;
FPointerColor := clWhite;
SetBounds(Left,Top,80,120);
Color := clBlack;
FSpacing := 4;
FShowZAxis := True;
FShowButtons := True;
if HasParent and not FCaptured then
	begin
	joyReleaseCapture(joystickid1);
	joySetCapture(handle,joystickid1,FPollingInterval,True);
	FCaptured := True;
	end;
end;

destructor TFlightJoyStick.Destroy;
begin
	joyReleaseCapture(joystickid1);
	inherited Destroy;
end;


procedure TFlightJoyStick.SetParent(AParent: TWinControl);
begin
inherited;
if HasParent and not FCaptured  and not (csDesigning	in componentstate) then
	begin
	joyReleaseCapture(joystickid1);
	joySetCapture(handle,joystickid1,FPollingInterval,True);
	FCaptured := True;
	ReadJoyStick;
	end;
end;


procedure TFlightJoyStick.MouseMove(Shift: TShiftState; X, Y: Integer);
var
	LPoint: TPoint;
begin
inherited;
if ssleft in Shift then
	if FCurrentMode = jcXYMouse then
		begin
		FCurrentMode := jcXYMouse;
		LPoint := ClientToScreen(point(X,Y));
		FJoyStickInfo.X_Axis := (LPoint.X / Screen.Width)* 2 - 1;
		FJoyStickInfo.Y_Axis := (LPoint.Y / Screen.Height)* 2 - 1;
		PaintXPointer(False);
		PaintYPointer(False);
		if assigned(FOnXYChange) then FOnXYChange(self, FJoyStickInfo);
		end
	else if FCurrentMode = jcZMouse then
		begin
		LPoint := ClientToScreen(point(X,Y));
		FJoyStickInfo.Z_Axis := (LPoint.Y / Screen.Height)* - 2 + 1;
		if FShowZAxis then PaintZPointer(False);
		if assigned(FOnZChange) then FOnZChange(self, FJoyStickInfo);
		end;
end;


procedure TFlightJoyStick.MouseDown(Button: TMouseButton;	Shift: TShiftState; X, Y: Integer);
var
	LPoint: TPoint;
begin
inherited;
if Button = mbLeft then
	begin
	LPoint := Point(X,Y);
	if PtInRect(FJoyEnabledRect,LPoint) then StickEnabled := not StickEnabled
	else if PtInRect(FXYAxisRect,LPoint) then
		begin
		FOldCursorPos := ClientToScreen(LPoint);
		SetCursorPos(round((FJoyStickInfo.X_Axis + 1) * (Screen.Width div 2)),round((FJoyStickInfo.Y_Axis + 1) * (Screen.Height div 2)));
		FCurrentMode := jcXYMouse;
		ShowCursor(False);
		SetCapture(Handle);
		end
	else if PtInRect(FZAxisRect,LPoint) then
		begin
		FOldCursorPos := ClientToScreen(LPoint);
		SetCursorPos(round((FJoyStickInfo.X_Axis + 1) * (Screen.Width div 2)),round((2 - (FJoyStickInfo.Z_Axis + 1)) * (Screen.Height div 2)));
		FCurrentMode := jcZMouse;
		ShowCursor(False);
		SetCapture(Handle);
		end
	else if PtInRect(FButton1Rect,LPoint) then
		begin
		if FStickyButtons then FJoyStickInfo.Button1 := not FJoyStickInfo.Button1
		else FJoyStickInfo.Button1 := True;
		if FStickyButtons then FCurrentMode := jcNone
		else FCurrentMode := jcbutton1;
		Paintbutton(1,FJoyStickInfo.Button1);
		ButtonEvent(1,FJoyStickInfo.Button1);
		end
	else if PtInRect(FButton2Rect,LPoint) then
		begin
		if FStickyButtons then FJoyStickInfo.Button2 := not FJoyStickInfo.Button2
		else FJoyStickInfo.Button2 := True;
		if FStickyButtons then FCurrentMode := jcNone
		else FCurrentMode := jcbutton2;
		Paintbutton(2,FJoyStickInfo.Button2);
		ButtonEvent(2,FJoyStickInfo.Button2);
		end
	else if PtInRect(FButton3Rect,LPoint) then
		begin
		if FStickyButtons then FJoyStickInfo.Button3 := not FJoyStickInfo.Button3
		else FJoyStickInfo.Button3 := True;
		if FStickyButtons then FCurrentMode := jcNone
		else FCurrentMode := jcbutton3;
		Paintbutton(3,FJoyStickInfo.Button3);
		ButtonEvent(3,FJoyStickInfo.Button3);
		end
	else if PtInRect(FButton4Rect,LPoint) then
		begin
		if FStickyButtons then FJoyStickInfo.Button4 := not FJoyStickInfo.Button4
		else FJoyStickInfo.Button4 := True;
		if FStickyButtons then FCurrentMode := jcNone
		else FCurrentMode := jcbutton4;
		Paintbutton(4,FJoyStickInfo.Button4);
		ButtonEvent(4,FJoyStickInfo.Button4);
		end
	end;
end;


procedure TFlightJoyStick.MouseUp(Button: TMouseButton;	Shift: TShiftState; X, Y: Integer);
begin
inherited;
if Button = mbLeft then
	begin
	Case FCurrentMode of
		jcXYMouse,jcZMouse:
			begin
			ReleaseCapture;
			SetCursorPos(FOldCursorPos.X,FOldCursorPos.Y);
			Cursor := crDefault;
			ShowCursor(True);
			end;
		jcbutton1:
			begin
			FJoyStickInfo.Button1 := False;
			Paintbutton(1,FJoyStickInfo.Button1);
			if assigned(FOnButton1Up) then FOnButton1Up(self);
			end;
		jcbutton2:
			begin
			FJoyStickInfo.Button2 := False;
			Paintbutton(2,FJoyStickInfo.Button2);
			if assigned(FOnButton2Up) then FOnButton2Up(self);
			end;
		jcbutton3:
			begin
			FJoyStickInfo.Button3 := False;
			Paintbutton(3,FJoyStickInfo.Button3);
			if assigned(FOnButton3Up) then FOnButton3Up(self);
			end;
		jcbutton4:
			begin
			FJoyStickInfo.Button4 := False;
			Paintbutton(4,FJoyStickInfo.Button4);
			if assigned(FOnButton4Up) then FOnButton4Up(self);
			end;

		end;
	FCurrentMode := jcStick;
	end;
end;


procedure TFlightJoysTick.ButtonEvent(PButtonID: Integer; PDown: Boolean);
begin
if PDown then
	begin
	if (PButtonID = 1) and assigned(FOnButton1Down) then FOnButton1Down(self)
	else if (PButtonID = 2) and assigned(FOnButton2Down) then FOnButton2Down(self)
	else if (PButtonID = 3) and assigned(FOnButton3Down) then FOnButton3Down(self)
	else if (PButtonID = 4) and assigned(FOnButton4Down) then FOnButton4Down(self);
	end
else
	begin
	if (PButtonID = 1) and assigned(FOnButton1Up) then FOnButton1Up(self)
	else if (PButtonID = 2) and assigned(FOnButton2Up) then FOnButton2Up(self)
	else if (PButtonID = 3) and assigned(FOnButton3Up) then FOnButton3Up(self)
	else if (PButtonID = 4) and assigned(FOnButton4Up) then FOnButton4Up(self);
	end;
end;


procedure TFlightJoyStick.paint;
var
i: integer;
begin
PaintStickEnabled(FStickEnabled);

if FShowZAxis then
	begin
	canvas.Pen.color := FZRulerColor;
	i := FZAxisRect.Top;
	while i < FZAxisRect.Bottom do
		begin
		canvas.moveTo(FZAxisRect.Left + ARROW_SIZE,i);
		canvas.lineto(FZAxisRect.Right,i);
		inc(i,FSpacing);
		end;
	end;
	canvas.Pen.color := FRulerColor;

i:= FXAxisRect.Left;
while i < FXAxisRect.Right do
	begin
	Canvas.MoveTo(i,FXAxisRect.Top + ARROW_SIZE);
	Canvas.LineTo(i,FXAxisRect.Bottom);
	inc(i,FSpacing);
	end;

i := FYAxisRect.Top;
while i < FYAxisRect.Bottom do
	begin
	Canvas.MoveTo(FYAxisRect.Left,i);
	Canvas.LineTo(FYAxisRect.Right,i);
	inc(i,FSpacing);
	end;
	PaintJoystickInfo;
end;


procedure TFlightJoyStick.PaintStickEnabled(PEnabled: Boolean);
begin
if PEnabled then Canvas.Brush.color := FEnabledColor
else Canvas.Brush.color := Color;
Canvas.Pen.Color := FRulerColor;
Canvas.Rectangle(FJoyEnabledRect.Left,FJoyEnabledRect.Top,FJoyEnabledRect.Right,FJoyEnabledRect.Bottom);
end;

procedure TFlightJoyStick.PaintButton(PID: Integer; PON: Boolean);
var
	LRect: TRect;
begin
if FShowButtons then
	begin
	if PID = 1 then LRect := FButton1Rect
	else if PID = 2 then LRect := FButton2Rect
	else if PID = 3 then LRect := FButton3Rect
	else if PID = 4 then LRect := FButton4Rect;
	Canvas.Pen.Color := FButtonEdgeColor;
	if PON then Canvas.Brush.Color := FButtonColor
	else Canvas.Brush.Color := Color;
	Canvas.ellipse(LRect.Left,LRect.Top,LRect.Right, LRect.Bottom);
	end;
end;


procedure TFlightJoyStick.PaintLeftArrow(PPoint: Tpoint);
var
	i,x,y: Integer;
begin
x := PPoint.X;
y := PPoint.Y;
for i := 0 to ARROW_SIZE do
	begin
	Canvas.MoveTo(x + i,y - i);
	Canvas.LineTo(x + i,y + i);
	end;
end;


procedure TFlightJoyStick.PaintRightArrow(PPoint: Tpoint);
var
	i,x,y: Integer;
begin
x := PPoint.X;
y := PPoint.Y;
for i := 0 to ARROW_SIZE do
	begin
	Canvas.MoveTo(x - i,y - i);
	Canvas.LineTo(x - i,y + i);
	end;
end;


procedure TFlightJoyStick.PaintDownArrow(PPoint: Tpoint);
var
	i,x,y: Integer;
begin
x := PPoint.X;
y := PPoint.Y;
for i := 0 to ARROW_SIZE do
	begin
	Canvas.MoveTo(x - i,y - i);
	Canvas.LineTo(x + i,y - i);
	end;
end;


procedure TFlightJoyStick.EraseDownArrow(PPoint: Tpoint);
var
	x,y: Integer;
begin
x := PPoint.X;
y := PPoint.Y;
canvas.brush.Color := Color;
Canvas.fillrect(rect(x-ARROW_SIZE,y - ARROW_SIZE,x + ARROW_SIZE,y + 1));
end;


procedure TFlightJoyStick.EraseRightArrow(PPoint: Tpoint);
var
	x,y: Integer;
begin
x := PPoint.X;
y := PPoint.Y;
canvas.brush.Color := Color;
Canvas.fillrect(rect(x-ARROW_SIZE,y - ARROW_SIZE,x + 1,y + ARROW_SIZE));
end;

procedure TFlightJoyStick.EraseLeftArrow(PPoint: Tpoint);
var
	x,y: Integer;
begin
x := PPoint.X;
y := PPoint.Y;
canvas.brush.Color := Color;
Canvas.fillrect(rect(x ,y - ARROW_SIZE,x+ARROW_SIZE + 1,y + ARROW_SIZE));
end;


procedure TFlightJoyStick.MMJOY1BUTTONDOWN(var Message: TMessage);
begin
if (message.WParam and JOY_BUTTON1 = JOY_BUTTON1) then
	begin
	if FStickyButtons then FJoyStickInfo.Button1 := not FJoyStickInfo.Button1
	else FJoyStickInfo.Button1 := True;
	PaintButton(1,FJoyStickInfo.Button1);
	ButtonEvent(1,FJoyStickInfo.Button1);
	end;
if (message.WParam and JOY_BUTTON2	 = JOY_BUTTON2) then
	begin
	if FStickyButtons then FJoyStickInfo.Button2 := not FJoyStickInfo.Button2
	else FJoyStickInfo.Button2 := True;
	PaintButton(2,FJoyStickInfo.Button2);
	ButtonEvent(2,FJoyStickInfo.Button2);
	end;
if (message.WParam and JOY_BUTTON3	 = JOY_BUTTON3) then
	begin
	if FStickyButtons then FJoyStickInfo.Button3 := not FJoyStickInfo.Button3
	else FJoyStickInfo.Button3 := True;
	PaintButton(3,FJoyStickInfo.Button3);
	ButtonEvent(3,FJoyStickInfo.Button3);
	end;
if (message.WParam and JOY_BUTTON4	 = JOY_BUTTON4) then
	begin
	if FStickyButtons then FJoyStickInfo.Button4 := not FJoyStickInfo.Button4
	else FJoyStickInfo.Button4 := True;
	PaintButton(4,FJoyStickInfo.Button4);
	ButtonEvent(4,FJoyStickInfo.Button4);
	end;
end;


procedure TFlightJoyStick.MMJOY1BUTTONUP(var Message: TMessage);
begin
if (message.WParam and JOY_BUTTON1CHG	 = JOY_BUTTON1CHG) then
	begin
	if not FStickyButtons then FJoyStickInfo.Button1 := False;
	PaintButton(1,FJoyStickInfo.Button1);
	ButtonEvent(1,FJoyStickInfo.Button1);
	end;
if (message.WParam and JOY_BUTTON2CHG	 = JOY_BUTTON2CHG) then
	begin
	if not FStickyButtons then FJoyStickInfo.Button2 := False;
	PaintButton(2,FJoyStickInfo.Button2);
	ButtonEvent(2,FJoyStickInfo.Button2);
	end;
if (message.WParam and JOY_BUTTON3CHG	 = JOY_BUTTON3CHG) then
	begin
	if not FStickyButtons then FJoyStickInfo.Button3 := False;
	PaintButton(3,FJoyStickInfo.Button3);
	ButtonEvent(3,FJoyStickInfo.Button3);
	end;
if (message.WParam and JOY_BUTTON4CHG	 = JOY_BUTTON4CHG) then
	begin
	if not FStickyButtons then FJoyStickInfo.Button4 := False;
	PaintButton(4,FJoyStickInfo.Button4);
	ButtonEvent(4,FJoyStickInfo.Button4);
	end;
end;


procedure TFlightJoyStick.MMJOY1ZMOVE(var Message: TMessage);
begin
if (FCurrentMode = jcStick) and FStickEnabled then
	begin
	FJoyStickInfo.Z_Axis := (message.LParamLo / Fjoycapability.wZmax)* -2 + 1;
	if FShowZAxis then PaintZPointer(False);
	if assigned(FOnZChange) then FOnZChange(self, FJoyStickInfo);
	end;
end;


procedure TFlightJoyStick.MMJOY1MOVE(var Message: TMessage);
begin
if (FCurrentMode = jcStick) and FStickEnabled then
	begin
	FJoyStickInfo.Y_Axis := (message.LParamHi / Fjoycapability.wymax)* 2 - 1;
	PaintYPointer(False);
	FJoyStickInfo.X_Axis := (message.LParamLo / Fjoycapability.wxmax)* 2 - 1;
	PaintXPointer(False);
	if assigned(FOnXYChange) then FOnXYChange(self, FJoyStickInfo);
	end;
end;



procedure TFlightJoyStick.PaintXPointer(PForcePaint: Boolean);
var
	LXPointer: Integer;
begin
LXPointer := FXAxisRect.Left + trunc((FJoyStickInfo.X_Axis + 1) * ((FXAxisRect.right - FXAxisRect.Left) div 2));
if (LXPointer <> FXCursor) or PForcePaint then
	begin
	EraseDownArrow(point(FXCursor,5));
	Canvas.pen.color := FPointerColor;
	PaintDownArrow(point(LXPointer,5));
	FXCursor := LXPointer;
	end;
end;

procedure TFlightJoyStick.PaintYPointer(PForcePaint: Boolean);
var
	LYPointer: Integer;
begin
LYPointer := FYAxisRect.Top + trunc((FJoyStickInfo.Y_Axis + 1) * ((FYAxisRect.Bottom - FYAxisRect.Top) div 2));
if (LYPointer <> FYCursor) or PForcePaint then
	begin
	EraseRightArrow(point(FYAxisRect.Left - 1,FYCursor));
	EraseLeftArrow(point(FYAxisRect.Right,FYCursor));
	Canvas.pen.color := FPointerColor;
	PaintRightArrow(point(FYAxisRect.Left - 1,LYPointer));
	PaintLeftArrow(point(FYAxisRect.Right,LYPointer));
	FYCursor := LYPointer;
	end;
end;


procedure TFlightJoyStick.PaintZPointer(PForcePaint: Boolean);
var
	LZPointer: Integer;
begin
LZPointer := FZAxisRect.Bottom - trunc((FJoyStickInfo.Z_Axis + 1) * ((FZAxisRect.Bottom - FZAxisRect.Top) div 2));
if (LZPointer <> FZCursor) or PForcePaint then
	begin
	EraseRightArrow(point(FZAxisRect.Left + ARROW_SIZE - 1,FZCursor));
	Canvas.pen.color := FZPointerColor;
	PaintRightArrow(point(FZAxisRect.Left + ARROW_SIZE - 1,LZPointer));
	FZCursor := LZPointer;
	end;
end;

procedure TFlightJoyStick.WMSIZE(var Message: TWMSIZE);
begin
CalcAreas;
end;


procedure TFlightJoyStick.CalcAreas;
const
	BTN_SIZE: integer = 8;
	LINE_SIZE: Integer = 4;
var
	LClientRect: TRect;
	LSpacing: Integer;
begin
//Calculate Button Area
if FShowButtons then
	begin
	LClientRect := rect(2,Height - (BTN_SIZE + 4),Width - 4,Height - 2);
	if FShowZAxis then LClientRect.Left := LClientRect.Left + 14;
	end
else
	LClientRect := rect(2,Height - 2,Width - 4,Height - 2);

//Calc Button Rectangles
LSpacing := ((LClientRect.Right - LClientRect.Left) - (4 * BTN_SIZE)) div 3;

FButton1Rect := Rect(LClientRect.Left,LClientRect.Top,LClientRect.Left + BTN_SIZE,LClientRect.Bottom);
FButton2Rect := Rect(FButton1Rect.Right + LSpacing,LClientRect.Top,FButton1Rect.Right + LSpacing + BTN_SIZE,LClientRect.Bottom);
FButton3Rect := Rect(FButton2Rect.Right + LSpacing,LClientRect.Top,FButton2Rect.Right + LSpacing + BTN_SIZE,LClientRect.Bottom);
FButton4Rect := Rect(FButton3Rect.Right + LSpacing,LClientRect.Top,FButton3Rect.Right + LSpacing + BTN_SIZE,LClientRect.Bottom);

//Calc X Axis area
FXAxisRect := rect(MARGIN,MARGIN,width - MARGIN,MARGIN + ARROW_SIZE + MARKER_SIZE);
if FShowZAxis then FXAxisRect.Left := MARGIN + ARROW_SIZE + MARKER_SIZE;

//Calc Y Axis area
LSpacing := (FXAxisRect.Right - FXAxisRect.Left) div 2;
FYAxisRect := rect(FXAxisRect.Left + LSpacing - (MARKER_SIZE div 2),FXAxisRect.Bottom + ARROW_SIZE,FXAxisRect.Left + LSpacing + (MARKER_SIZE div 2),LClientRect.Top - ARROW_SIZE);


//Calc XY Axis area
FXYAxisRect := Rect(FXAxisRect.Left,FYAxisRect.Top,FXAxisRect.Right,FYAxisRect.Bottom);

//Calc Z Axis Area
if FShowZAxis then
	FZAxisRect := Rect(MARGIN,FXAxisRect.Bottom,MARGIN + ARROW_SIZE + MARKER_SIZE,Height - MARGIN - ARROW_SIZE);

//Calc Joystick Disable Button Area
//	FJoyEnabledRect := rect(MARGIN,MARGIN,MARGIN + ARROW_SIZE + MARKER_SIZE, MARGIN + ARROW_SIZE + MARKER_SIZE);
FJoyEnabledRect := Rect(FXYAxisRect.Right - BTN_SIZE,FXYAxisRect.Bottom - BTN_SIZE,FXYAxisRect.Right,FXYAxisRect.Bottom);
end;


procedure TFlightJoyStick.SetPollingInterval(Pvalue: Integer);
begin
FPollingInterval := PValue;
if PValue <> FPollingInterval then
	begin
	FPollingInterval := PValue;
	joyReleaseCapture(joystickid1);
	if not (csDesigning	in componentstate) then
		joySetCapture(handle,joystickid1,FPollingInterval,True);
	end;
end;


procedure TFLightJoyStick.SetAutoPolling(PValue: Boolean);
begin
if PValue <> FAutoPolling then
	begin
	FAutoPolling := PValue;
	if FAutoPolling then
		begin
		if not (csDesigning	in componentstate) then
			joySetCapture(handle,joystickid1,FPollingInterval,True);
		end
	else
		begin
		joyReleaseCapture(joystickid1);
		end;
	end;
end;


procedure TFLightJoyStick.SetShowZAxis(PValue: Boolean);
begin
if PValue <> FShowZAxis then
	begin
	FShowZAxis := PValue;
	CalcAreas;
	invalidate;
	end;
end;


procedure TFlightJoyStick.SetSpacing(PValue: Integer);
begin
if PValue <> FSpacing then
	begin
	if PValue < 2 then PValue := 2;
	FSpacing := PValue;
	Invalidate;
	end;
end;

procedure TFlightJoyStick.SetShowButtons(PValue: boolean);
begin
if PValue <> FShowButtons then
	begin
	FShowButtons := PValue;
	CalcAreas;
	Invalidate;
	end;
end;


procedure TFlightJoyStick.SetZRulerColor(PValue: TColor);
begin
if PValue <> FZRulerColor then
	begin
	FZRulerColor := PValue;
	Invalidate;
	end;
end;


procedure TFlightJoyStick.SetRulerColor(Pvalue: TColor);
begin
if PValue <> FRulerColor then
	begin
	FRulerColor := PValue;
	Invalidate;
	end;
end;


procedure TFlightJoyStick.SetEnabledColor(Pvalue: TColor);
begin
if PValue <> FEnabledColor then
	begin
	FEnabledColor := PValue;
	Invalidate;
	end;
end;


procedure TFlightJoyStick.SetButtonColor(Pvalue: TColor);
begin
if PValue <> FButtonColor then
	begin
	FButtonColor := PValue;
	Invalidate;
	end;
end;


procedure TFlightJoyStick.SetButtonEdgeColor(PValue: TColor);
begin
if PValue <> FButtonEdgeColor then
	begin
	FButtonEdgeColor := PValue;
	Invalidate;
	end;
end;

procedure TFlightJoyStick.SetZPointerColor(PValue: TColor);
begin
if PValue <> FZPointerColor then
	begin
	FZPointerColor := PValue;
	Invalidate;
	end;
end;


procedure TFlightJoyStick.SetPointerColor(Pvalue: TColor);
begin
if PValue <> FPointerColor then
	begin
	FPointerColor := PValue;
	Invalidate;
	end;
end;


function TFlightJoyStick.GetXAxis: Single;
begin
if not FAutoPolling then ReadJoyStick;
Result := FJoyStickInfo.X_Axis
end;

function TFlightJoyStick.GetYAxis: Single;
begin
if not FAutoPolling then ReadJoyStick;
Result := FJoyStickInfo.Y_Axis
end;


function TFlightJoyStick.GetZAxis: Single;
begin
if not FAutoPolling then ReadJoyStick;
Result := FJoyStickInfo.Z_Axis
end;

function TFlightJoyStick.GetButton1: Boolean;
begin
if not FAutoPolling then ReadJoyStick;
Result := FJoyStickInfo.Button1;
end;

function TFlightJoyStick.GetButton2: Boolean;
begin
if not FAutoPolling then ReadJoyStick;
Result := FJoyStickInfo.Button2;
end;


function TFlightJoyStick.GetButton3: Boolean;
begin
if not FAutoPolling then ReadJoyStick;
Result := FJoyStickInfo.Button3;
end;


function TFlightJoyStick.GetButton4: Boolean;
begin
if not FAutoPolling then ReadJoyStick;
Result := FJoyStickInfo.Button4;
end;


procedure TFlightJoyStick.ReadJoyStick;
var
	LJoy: tjoyinfo;
begin
if FStickEnabled then
	begin
	joygetpos(joystickid1,@LJoy);
	//Read all thre axis
	if LJoy.wxpos = 0 then FJoyStickInfo.X_Axis := -1
	else FJoyStickInfo.X_Axis := (LJoy.wxpos / Fjoycapability.wXmax)* 2 - 1;
	if LJoy.wypos = 0 then FJoyStickInfo.Y_Axis := -1
	else FJoyStickInfo.Y_Axis := (LJoy.wypos / Fjoycapability.wymax)* 2 - 1;
	if LJoy.wzpos = 0 then FJoyStickInfo.Z_Axis := 1
	else FJoyStickInfo.Z_Axis := (LJoy.wzpos / Fjoycapability.wzmax)* -2 + 1;
	//Read all buttons
	FButtons := LJoy.wButtons;
	FJoyStickInfo.Button1 := LJoy.wButtons and JOY_BUTTON1 = JOY_BUTTON1;
	FJoyStickInfo.Button2 := LJoy.wButtons and JOY_BUTTON2 = JOY_BUTTON2;
	FJoyStickInfo.Button3 := LJoy.wButtons and JOY_BUTTON3 = JOY_BUTTON3;
	FJoyStickInfo.Button4 := LJoy.wButtons and JOY_BUTTON4 = JOY_BUTTON4;
  end;
end;

function TFlightJoyStick.GetJoyStickInfo: TJoyStickInfo;
begin
if not FAutoPolling then ReadJoyStick;
Result := FJoyStickInfo;
PaintJoystickInfo;
end;


procedure TFlightJoyStick.PaintJoyStickInfo;
begin
if FShowButtons then
	begin
	PaintButton(1,FJoyStickInfo.Button1);
	PaintButton(2,FJoyStickInfo.Button2);
	PaintButton(3,FJoyStickInfo.Button3);
	PaintButton(4,FJoyStickInfo.Button4);
	end;
PaintXPointer(True);
PaintYPointer(True);
if FShowZAxis then PaintZPointer(True);
end;


procedure TFlightJoyStick.SetStickEnabled(PValue: Boolean);
begin
if PValue <> FStickEnabled then
	begin
	FStickEnabled := PValue;
	Invalidate;
	end;
end;


end.
